/*
** char_translate.c for Xclamation, XQuad and XAllWrite in Global/
**
** Copyright (C) 1994-2000 Axene.
** Authors: Stphane Boisson, Antoine Buat, Robin Castanier and Emmanuel Paris.
** Email: xcalibur@axene.org
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** Started on  Mon Jul 17 14:15:33 1995 Emmanuel Paris
** Last update Tue Mar 24 16:22:22 1998 Emmanuel Paris
*/

#include "char_translate.h"
#include "encoding_sym.h"

unsigned char ct_suppress_accent[256] = 
{
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 XcS_A,
 XcS_B,
 XcS_C,
 XcS_D,
 XcS_E,
 XcS_F,
 XcS_G,
 XcS_H,
 XcS_I,
 XcS_J,
 XcS_K,
 XcS_L,
 XcS_M,
 XcS_N,
 XcS_O,
 XcS_P,
 XcS_Q,
 XcS_R,
 XcS_S,
 XcS_T,
 XcS_U,
 XcS_V,
 XcS_W,
 XcS_X,
 XcS_Y,
 XcS_Z,
 0, 0, 0, 0, 0, 0,
 XcS_a,
 XcS_b,
 XcS_c,
 XcS_d,
 XcS_e,
 XcS_f,
 XcS_g,
 XcS_h,
 XcS_i,
 XcS_j,
 XcS_k,
 XcS_l,
 XcS_m,
 XcS_n,
 XcS_o,
 XcS_p,
 XcS_q,
 XcS_r,
 XcS_s,
 XcS_t,
 XcS_u,
 XcS_v,
 XcS_w,
 XcS_x,
 XcS_y,
 XcS_z,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 XcS_A,
 XcS_A,
 XcS_A,
 XcS_A,
 XcS_A,
 XcS_A,
 XcS_A,
 XcS_C,
 XcS_E,
 XcS_E,
 XcS_E,
 XcS_E,
 XcS_I,
 XcS_I,
 XcS_I,
 XcS_I,
 0,
 XcS_N,
 XcS_O,
 XcS_O,
 XcS_O,
 XcS_O,
 XcS_O,
 0,
 XcS_O,
 XcS_U,
 XcS_U,
 XcS_U,
 XcS_U,
 XcS_Y,
 0,
 XcS_s,
 XcS_a,
 XcS_a,
 XcS_a,
 XcS_a,
 XcS_a,
 XcS_a,
 XcS_a,
 XcS_c,
 XcS_e,
 XcS_e,
 XcS_e,
 XcS_e,
 XcS_i,
 XcS_i,
 XcS_i,
 XcS_i,
 0,
 XcS_n,
 XcS_o,
 XcS_o,
 XcS_o,
 XcS_o,
 XcS_o,
 0,
 XcS_o,
 XcS_u,
 XcS_u,
 XcS_u,
 XcS_u,
 XcS_y,
 0,
 XcS_y
};

unsigned char ct_to_upper[256] = 
{
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 XcS_A,
 XcS_B,
 XcS_C,
 XcS_D,
 XcS_E,
 XcS_F,
 XcS_G,
 XcS_H,
 XcS_I,
 XcS_J,
 XcS_K,
 XcS_L,
 XcS_M,
 XcS_N,
 XcS_O,
 XcS_P,
 XcS_Q,
 XcS_R,
 XcS_S,
 XcS_T,
 XcS_U,
 XcS_V,
 XcS_W,
 XcS_X,
 XcS_Y,
 XcS_Z,
 0, 0, 0, 0, 0, 0,
 XcS_A,
 XcS_B,
 XcS_C,
 XcS_D,
 XcS_E,
 XcS_F,
 XcS_G,
 XcS_H,
 XcS_I,
 XcS_J,
 XcS_K,
 XcS_L,
 XcS_M,
 XcS_N,
 XcS_O,
 XcS_P,
 XcS_Q,
 XcS_R,
 XcS_S,
 XcS_T,
 XcS_U,
 XcS_V,
 XcS_W,
 XcS_X,
 XcS_Y,
 XcS_Z,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 XcS_Agrave,
 XcS_Aacute,
 XcS_Acircumflex,
 XcS_Atilde,
 XcS_Adieresis,
 XcS_Aring,
 XcS_AE,
 XcS_Ccedilla,
 XcS_Egrave,
 XcS_Eacute,
 XcS_Ecircumflex,
 XcS_Edieresis,
 XcS_Igrave,
 XcS_Iacute,
 XcS_Icircumflex,
 XcS_Idieresis,
 XcS_Eth,
 XcS_Ntilde,
 XcS_Ograve,
 XcS_Oacute,
 XcS_Ocircumflex,
 XcS_Otilde,
 XcS_Odieresis,
 0,
 XcS_Oslash,
 XcS_Ugrave,
 XcS_Uacute,
 XcS_Ucircumflex,
 XcS_Udieresis,
 XcS_Yacute,
 XcS_Thorn,
 0,
 XcS_Agrave,
 XcS_Aacute,
 XcS_Acircumflex,
 XcS_Atilde,
 XcS_Adieresis,
 XcS_Aring,
 XcS_AE,
 XcS_Ccedilla,
 XcS_Egrave,
 XcS_Eacute,
 XcS_Ecircumflex,
 XcS_Edieresis,
 XcS_Igrave,
 XcS_Iacute,
 XcS_Icircumflex,
 XcS_Idieresis,
 XcS_Eth,
 XcS_Ntilde,
 XcS_Ograve,
 XcS_Oacute,
 XcS_Ocircumflex,
 XcS_Otilde,
 XcS_Odieresis,
 0,
 XcS_Oslash,
 XcS_Ugrave,
 XcS_Uacute,
 XcS_Ucircumflex,
 XcS_Udieresis,
 XcS_Yacute,
 XcS_Thorn,
 XcS_Thorn,
 /*  0, */
};

unsigned char ct_to_lower[256] = 
{
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 XcS_a,
 XcS_b,
 XcS_c,
 XcS_d,
 XcS_e,
 XcS_f,
 XcS_g,
 XcS_h,
 XcS_i,
 XcS_j,
 XcS_k,
 XcS_l,
 XcS_m,
 XcS_n,
 XcS_o,
 XcS_p,
 XcS_q,
 XcS_r,
 XcS_s,
 XcS_t,
 XcS_u,
 XcS_v,
 XcS_w,
 XcS_x,
 XcS_y,
 XcS_z,
 0, 0, 0, 0, 0, 0,
 XcS_a,
 XcS_b,
 XcS_c,
 XcS_d,
 XcS_e,
 XcS_f,
 XcS_g,
 XcS_h,
 XcS_i,
 XcS_j,
 XcS_k,
 XcS_l,
 XcS_m,
 XcS_n,
 XcS_o,
 XcS_p,
 XcS_q,
 XcS_r,
 XcS_s,
 XcS_t,
 XcS_u,
 XcS_v,
 XcS_w,
 XcS_x,
 XcS_y,
 XcS_z,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 XcS_agrave,
 XcS_aacute,
 XcS_acircumflex,
 XcS_atilde,
 XcS_adieresis,
 XcS_aring,
 XcS_ae,
 XcS_ccedilla,
 XcS_egrave,
 XcS_eacute,
 XcS_ecircumflex,
 XcS_edieresis,
 XcS_igrave,
 XcS_iacute,
 XcS_icircumflex,
 XcS_idieresis,
 XcS_eth,
 XcS_ntilde,
 XcS_ograve,
 XcS_oacute,
 XcS_ocircumflex,
 XcS_otilde,
 XcS_odieresis,
 0,
 XcS_oslash,
 XcS_ugrave,
 XcS_uacute,
 XcS_ucircumflex,
 XcS_udieresis,
 XcS_yacute,
 XcS_thorn,
 0,
 XcS_agrave,
 XcS_aacute,
 XcS_acircumflex,
 XcS_atilde,
 XcS_adieresis,
 XcS_aring,
 XcS_ae,
 XcS_ccedilla,
 XcS_egrave,
 XcS_eacute,
 XcS_ecircumflex,
 XcS_edieresis,
 XcS_igrave,
 XcS_iacute,
 XcS_icircumflex,
 XcS_idieresis,
 XcS_eth,
 XcS_ntilde,
 XcS_ograve,
 XcS_oacute,
 XcS_ocircumflex,
 XcS_otilde,
 XcS_odieresis,
 0,
 XcS_oslash,
 XcS_ugrave,
 XcS_uacute,
 XcS_ucircumflex,
 XcS_udieresis,
 XcS_yacute,
 XcS_thorn,
 XcS_ydieresis
};

unsigned char ct_type[256] = 
{
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW,  
 Xc_PUNCT, 
 Xc_PUNCT,
 Xc_PUNCT,
 Xc_MISC,
 Xc_CURRENCY,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_PUNCT,
 Xc_PUNCT,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_DIGIT,
 Xc_PUNCT,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_PUNCT,
 Xc_MISC,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_PUNCT,
 Xc_MISC,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_MISC,
 Xc_PUNCT,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, Xc_UNKNOW, 
 Xc_PUNCT,
 Xc_PUNCT,
 Xc_CURRENCY,
 Xc_CURRENCY,
 Xc_CURRENCY,
 Xc_CURRENCY,
 Xc_MISC,
 Xc_PUNCT,
 Xc_MISC,
 Xc_MISC,
 Xc_MISC,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_MISC,		/* hyphen ... */
 Xc_MISC,
 Xc_MISC,		/* macron ... */
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_MISC,
 Xc_OPERATOR,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_MISC,
 Xc_OPERATOR,
 Xc_MISC,
 Xc_PUNCT,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_OPERATOR,
 Xc_PUNCT,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_OPERATOR,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_OPERATOR,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER,
 Xc_LETTER
};

boolean ct_breakable[256] = 
{
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE,  
 TRUE, 
 TRUE,
 FALSE,		/* " */
 FALSE,
 FALSE,
 FALSE,
 TRUE,
 FALSE,
 FALSE,
 TRUE,
 TRUE,
 TRUE,
 TRUE,
 TRUE,
 TRUE,
 TRUE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 TRUE,
 TRUE,
 TRUE,
 TRUE,
 TRUE,
 TRUE,		/* ? */
 FALSE,
 FALSE,		/* A */
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,		/* Z */
 FALSE,
 FALSE,
 TRUE,
 TRUE,
 FALSE,
 FALSE,
 FALSE,		/* a */
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,		/* z */
 FALSE,
 TRUE,
 TRUE,		/* brace right */
 TRUE,		/* tidle */
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, 
 TRUE, TRUE, TRUE, TRUE, TRUE,
 FALSE,		/* nobreakspace */
 TRUE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 TRUE,
 TRUE,		/* hyphen ... */
 FALSE,
 TRUE,		/* macron ... */
 TRUE,
 TRUE,
 TRUE,
 TRUE,
 FALSE,
 TRUE,
 TRUE,
 TRUE,
 FALSE,
 TRUE,
 FALSE,
 TRUE,
 TRUE,		/* one quarter */
 TRUE,
 TRUE,
 TRUE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 TRUE,		/* multiply */
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 TRUE,		/* divide */
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE,
 FALSE
};

boolean Xc_strcmp_parser(s1, s2)
char	*s1;
char	*s2;
{
 char	c1, c2;
  
 while(*s1 && *s2)
 {
  c1 = Xc_TO_LOWER(*s1);
  c2 = Xc_TO_LOWER(*s2);
      
  if (c1 != c2 && c1 != Xc_SUPPRESS_ACCENT(c2))
   return FALSE;
      
  s1++; s2++;
 }
  
 return (*s1 == *s2);
}









